==> jimmer-spring-boot-starter-0.7.24-sources.jar: org/babyfish/jimmer/spring/cloud/SpringCloudExchange.java <==
package org.babyfish.jimmer.spring.cloud;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.babyfish.jimmer.meta.ImmutableProp;
import org.babyfish.jimmer.runtime.ImmutableSpi;
import org.babyfish.jimmer.sql.ast.tuple.Tuple2;
import org.babyfish.jimmer.sql.fetcher.Fetcher;
import org.babyfish.jimmer.sql.runtime.MicroServiceExchange;
import org.springframework.web.client.RestTemplate;

import java.util.Collection;
import java.util.List;

public class SpringCloudExchange implements MicroServiceExchange {

    private final RestTemplate restTemplate;

    private final ObjectMapper mapper;

    public SpringCloudExchange(RestTemplate restTemplate, ObjectMapper mapper) {
        this.restTemplate = restTemplate;
        this.mapper = mapper;
    }

    @SuppressWarnings("unchecked")
    @Override
    public List<ImmutableSpi> findByIds(
            String microServiceName,
            Collection<?> ids,
            Fetcher<?> fetcher
    ) throws JsonProcessingException {
        return restTemplate.getForObject(
                "http://{microServiceName}/jimmerMicroServiceBridge/byIds" +
                        "?ids={ids}&fetcher={fetcher}",
                List.class,
                microServiceName,
                mapper.writeValueAsString(ids),
                fetcher.toString(true)
        );
    }

    @SuppressWarnings("unchecked")
    @Override
    public List<Tuple2<Object, ImmutableSpi>> findByAssociatedIds(
            String microServiceName,
            ImmutableProp prop,
            Collection<?> targetIds,
            Fetcher<?> fetcher
    ) throws JsonProcessingException {
        return restTemplate.getForObject(
                "http://{microServiceName}/jimmerMicroServiceBridge/byAssociatedIds" +
                        "?prop={prop}&targetIds={targetIds}&fetcher={fetcher}",
                List.class,
                microServiceName,
                prop.getName(),
                mapper.writeValueAsString(targetIds),
                fetcher.toString(true)
        );
    }
}

==> jimmer-spring-boot-starter-0.7.25-sources.jar: org/babyfish/jimmer/spring/cloud/SpringCloudExchange.java <==
package org.babyfish.jimmer.spring.cloud;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.type.TypeFactory;
import org.babyfish.jimmer.impl.util.Classes;
import org.babyfish.jimmer.meta.ImmutableProp;
import org.babyfish.jimmer.runtime.ImmutableSpi;
import org.babyfish.jimmer.sql.ast.tuple.Tuple2;
import org.babyfish.jimmer.sql.fetcher.Fetcher;
import org.babyfish.jimmer.sql.runtime.MicroServiceExchange;
import org.springframework.web.client.RestTemplate;

import java.util.Collection;
import java.util.List;

public class SpringCloudExchange implements MicroServiceExchange {

    private final RestTemplate restTemplate;

    private final ObjectMapper mapper;

    public SpringCloudExchange(RestTemplate restTemplate, ObjectMapper mapper) {
        this.restTemplate = restTemplate;
        this.mapper = mapper;
    }

    @Override
    public List<ImmutableSpi> findByIds(
            String microServiceName,
            Collection<?> ids,
            Fetcher<?> fetcher
    ) throws JsonProcessingException {
        String json = restTemplate.getForObject(
                "http://{microServiceName}/jimmerMicroServiceBridge/byIds" +
                        "?ids={ids}&fetcher={fetcher}",
                String.class,
                microServiceName,
                mapper.writeValueAsString(ids),
                fetcher.toString(true)
        );
        return mapper.readValue(
                json,
                mapper.getTypeFactory().constructParametricType(
                        List.class,
                        fetcher.getImmutableType().getJavaClass()
                )
        );
    }

    @Override
    public List<Tuple2<Object, ImmutableSpi>> findByAssociatedIds(
            String microServiceName,
            ImmutableProp prop,
            Collection<?> targetIds,
            Fetcher<?> fetcher
    ) throws JsonProcessingException {
        String json = restTemplate.getForObject(
                "http://{microServiceName}/jimmerMicroServiceBridge/byAssociatedIds" +
                        "?prop={prop}&targetIds={targetIds}&fetcher={fetcher}",
                String.class,
                microServiceName,
                prop.getName(),
                mapper.writeValueAsString(targetIds),
                fetcher.toString(true)
        );
        TypeFactory typeFactory = mapper.getTypeFactory();
        return mapper.readValue(
                json,
                typeFactory.constructParametricType(
                        List.class,
                        typeFactory.constructParametricType(
                                Tuple2.class,
                                Classes.boxTypeOf(prop.getTargetType().getIdProp().getElementClass()),
                                fetcher.getImmutableType().getJavaClass()
                        )
                )
        );
    }
}
